"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const metadata_registry_1 = require("../metadata-registry");
const i18n_1 = require("../i18n");
const utils_1 = require("../utils");
// TODO: consolidate this with supported types in deploy
const supportedTypes = new Set([
    'ApexClass',
    'ApexTrigger',
    'ApexPage',
    'ApexComponent'
]);
class ToolingApi extends types_1.BaseApi {
    retrieveWithPaths(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const retrievePaths = options.paths[0];
            const registry = new metadata_registry_1.RegistryAccess();
            return yield this.retrieve({
                output: options.paths[0],
                components: registry.getComponentsFromPath(retrievePaths)
            });
        });
    }
    retrieve(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let retrieveResult;
            if (options.components.length > 1) {
                const retrieveError = new Error();
                retrieveError.message = i18n_1.nls.localize('tapi_retrieve_component_limit_error');
                retrieveError.name = 'MetadataRetrieveLimit';
                throw retrieveError;
            }
            const mdComponent = options.components[0];
            if (!supportedTypes.has(mdComponent.type.name)) {
                const retrieveError = new Error();
                retrieveError.message = i18n_1.nls.localize('beta_tapi_membertype_unsupported_error', mdComponent.type.name);
                retrieveError.name = 'MetadataTypeUnsupported';
                throw retrieveError;
            }
            try {
                const queryResult = (yield this.connection.tooling.query(this.buildQuery(mdComponent.type.name, mdComponent.fullName)));
                if (queryResult && queryResult.records.length === 0) {
                    return {
                        success: true,
                        components: [],
                        message: i18n_1.nls.localize('error_md_not_present_in_org', options.output)
                    };
                }
                // If output is defined it overrides where the component will be stored
                const mdSourcePath = options.output
                    ? options.output
                    : mdComponent.sources[0];
                const saveFilesMap = new Map();
                saveFilesMap.set(mdSourcePath, queryResult.records[0].Body);
                saveFilesMap.set(utils_1.generateMetaXMLPath(mdSourcePath), utils_1.generateMetaXML(mdComponent.type.name, queryResult.records[0].ApiVersion, queryResult.records[0].Status));
                utils_1.createFiles(saveFilesMap);
                retrieveResult = {
                    success: true,
                    components: [mdComponent]
                };
            }
            catch (err) {
                throw new Error(i18n_1.nls.localize('error_in_tooling_retrieve', err));
            }
            return retrieveResult;
        });
    }
    buildQuery(typeName, fullName) {
        return `Select Id, ApiVersion, Body, Name, NamespacePrefix, Status from ${typeName} where Name = '${fullName}'`;
    }
}
exports.ToolingApi = ToolingApi;
//# sourceMappingURL=toolingApi.js.map