"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const constants_1 = require("../constants");
const fs_1 = require("fs");
const base_1 = require("./base");
const errors_1 = require("../../errors");
/**
 * Handles types with a single content file with a matching file extension.
 * These tend to be the programmatic types.
 *
 * __Example Types__:
 *
 * ApexClass, ApexTrigger, ApexComponent
 *
 * __Example Structure__:
 *
 * ```text
 * foos/
 * ├── foobar.ext
 * ├── foobar.ext-meta.xml
 *```
 */
class MatchingContentFile extends base_1.BaseSourceAdapter {
    getMetadataXmlPath(pathToSource) {
        return `${pathToSource}${constants_1.META_XML_SUFFIX}`;
    }
    getSourcePaths(fsPath, isMetaXml) {
        let sourcePath;
        if (isMetaXml) {
            const path = fsPath.slice(0, fsPath.lastIndexOf(constants_1.META_XML_SUFFIX));
            if (fs_1.existsSync(path)) {
                sourcePath = path;
            }
        }
        else {
            const suffix = path_1.extname(fsPath).slice(1);
            if (this.registry.suffixes[suffix]) {
                sourcePath = fsPath;
            }
        }
        if (sourcePath) {
            return [sourcePath];
        }
        throw new errors_1.ExpectedSourceFilesError(this.type, fsPath);
    }
}
exports.MatchingContentFile = MatchingContentFile;
//# sourceMappingURL=matchingContentFile.js.map